import { MedianoteFormat, MedianoteAPI } from "medianote-format-sdk";
import { Serie, Episode, PlotObj, GenreObj, PartialDate, URLsObj, SerieObj, ID, Film, MediaType } from "medianote-entities";
import {
	TITLE_SELECTOR,
	ORIGINAL_TITLE_SELECTOR,
	GENRE_SELECTOR,
	POSTER_SELECTOR,
	FANART_SELECTOR,
	YEAR_SELECTOR,
	SEASONS_SELECTOR,
	PLOT_SELECTOR
} from "./selectors";

export class Format extends MedianoteFormat {
	constructor(api: MedianoteAPI) {
		super(api, "Guardaserie", ["guardaserieId"]);
	}

	async run(params: any): Promise<Serie | Film> {
		await this.api.loadURL("https://www.guardaserie.tube/" + params.guardaserieId + "/");
		await this.api.waitSelector(TITLE_SELECTOR);

		const title: string = this.api.getTextContent(TITLE_SELECTOR, true);
		const plot: PlotObj = { it: this.api.getTextContent(PLOT_SELECTOR, true) };
		const originalTitle: string = this.api
			.getTextContent(ORIGINAL_TITLE_SELECTOR, true)
			.replace("TITOLO ORIGINALE:", "")
			.trim();
		const genre: string = this.api
			.getTextContent(GENRE_SELECTOR, true)
			.replace("GENERE:", "")
			.trim();
		const genres: GenreObj = { [genre]: true };
		const poster: string = this.api.getSrc(POSTER_SELECTOR);
		const fanart: string = this.api.getSrc(FANART_SELECTOR);
		const year: string = this.api
			.getTextContent(YEAR_SELECTOR, true)
			.replace("ANNO:", "")
			.trim();
		const array: string[] = year.split("-");
		const releaseDate: PartialDate = new PartialDate({ year: parseInt(array[0].trim()) });
		const terminationDate: PartialDate = array[1].trim() !== "In Lavorazione" ? null : new PartialDate({ year: parseInt(array[1].trim()) });

		const seasons = {};
		const totalSeasons = this.api.querySelector(SEASONS_SELECTOR).children().length;

		const $ = this.api.get$();

		for (let seasonNumber = 1; seasonNumber <= totalSeasons; seasonNumber++) {
			const selector: string = "div.row-stagione-" + seasonNumber;
			const seasonNode = $(selector);

			const episodesArray: CheerioElement[] = seasonNode.find(".col-xs-6.col-sm-3").toArray();
			let episodeNumber: number = 0;
			const episodes = {};
			for (const node of episodesArray) {
				episodeNumber++;
				const episodeNode: Cheerio = $(node);

				const previewNode = episodeNode.find("div.content-image-screen img");
				let preview: string = previewNode.attr("data-original");
				if (preview === undefined || preview.length === 0) preview = previewNode.attr("meta-src");
				if (preview === undefined || preview.length === 0) preview = previewNode.attr("src");

				const episodeTitle: string = episodeNode
					.find("div.upper-title-thumb span")
					.text()
					.trim();
				const episodePlot: PlotObj = {
					it: episodeNode
						.find("div.title-serie-thumb-single p")
						.text()
						.trim()
				};

				const links = episodeNode.find("a > span");

				const providers = ["speedvideo", "mixdrop", "gounlimited", "supervideo", "vup"];

				const urls: URLsObj = {};
				for (let i = 1; i <= 5; i++) {
					const id: string = i != 1 ? i.toString() : "";
					const embed: string = links.attr("meta-embed" + id);
					if (embed !== "") urls[providers[i - 1]] = embed;
				}

				const episode: Episode = new Episode({
					number: episodeNumber,
					season: seasonNumber,
					title: episodeTitle,
					plot: episodePlot,
					preview,
					urls
				});
				episodes[episodeNumber] = episode;
			}
			seasons[seasonNumber] = episodes;
		}

		const serieObj: Partial<SerieObj> = {
			title,
			originalTitle,
			plot,
			poster,
			fanart,
			releaseDate,
			genres,
			seasons,
			type: MediaType.SERIE,
			extra: { guardaserieId: params.guardaserieId }
		};

		if (terminationDate) serieObj.terminationDate = terminationDate;

		return new Serie(serieObj);
	}
}
